@@ -1715,6 +1715,15 @@ do_set_attribute (GVfsBackend *backend,
 
   if (strcmp (attribute, G_FILE_ATTRIBUTE_TIME_MODIFIED) == 0)
     {
+      if (type != G_FILE_ATTRIBUTE_TYPE_UINT64) 
+        {
+          g_vfs_job_failed (G_VFS_JOB (job),
+                            G_IO_ERROR,
+                            G_IO_ERROR_INVALID_ARGUMENT,
+                            "%s",
+                            _("Invalid attribute type (uint64 expected)"));
+        }
+
       smbc_utimes = smbc_getFunctionUtimes (op_backend->smb_context);
       tbuf[1].tv_sec = (*(guint64 *)value_p);  /* mtime */
       tbuf[1].tv_usec = 0;
